@@ -29,6 +29,7 @@ from urllib3.util import Retry
 import time
 from functools import wraps
 import logging
+import pandas as pd
 from posixpath import join as urljoin
 from pathlib import Path
 from os import PathLike, path
@@ -42,6 +43,7 @@ from ._constants import *
 logger = logging.getLogger(__name__)
 logging.basicConfig(level=logging.DEBUG)
 
+# debug = True
 debug = False
 if debug is True:
     logger.setLevel(logging.DEBUG)
@@ -98,7 +100,7 @@ class IndexHandler:
         self.root_path = self._prepare_root_path(root_path)
         self._checked_index_creation = False
         self._base_index_path = self.root_path / "index" / "base_index"
-        self._file_num_index_path = self.root_path / "index" / "file_num_index"
+        self._num_index_path = self.root_path / "index" / "file_num_index"
 
 
     
@@ -194,53 +196,65 @@ class IndexHandler:
         if there are duplicate values in the base_index.
         Removes the duplicates and entries not present locally.'''
         # get all base indexes
-        import pandas as pd
         base_indexes = [p for p in self._base_index_path.glob("*.csv")]
         base_remove_count = 0
+        num_indexes = [p for p in self._num_index_path.glob("*.json")]
         num_remove_count = 0
-        for b in base_indexes:
-            original = pd.read_csv(b, delimiter=",")
-            # remove duplicates
-            df = original.drop_duplicates()
-            # load the file_num_index
-            num_index_path = self._file_num_index_path / b.name.replace(".csv", ".json")
-            with open(num_index_path, "r+") as file_num_index:
-                num_changed =  False
-                base_changed = False
-                num_index = json.load(file_num_index)
-                drop_rows = []
-                for row in df.iloc:
-                    if not (self.root_path / Path(row["file_path"])).exists():
-                        # remove entry from file_num_index
-                        try:
-                            num_obj = num_index[row["file_number"]]
-                            drop_idx = None
-                            for idx, item in enumerate(num_obj):
-                                if item[1] == row["file_path"]:
-                                    drop_idx = idx
-                                    break
-                            if drop_idx:
-                                num_changed = True
-                                num_obj.pop(drop_idx)
-                                num_index[row["file_number"]] = num_obj
-                                num_remove_count += 1
-                        except KeyError as e:
-                            logger.debug(f"{e} when trying to remove entry from file_number_index, row in base_index: {row}")
 
-                        base_changed = True
-                        drop_rows.append(row.name)    
-                # remove invalid rows from dataframe/base_index and rewrite files if needed
-                if base_changed is True:
-                    base_remove_count += len(drop_rows)
-                    df = df.drop(drop_rows)
-                    df.to_csv(b)
-                    logger.debug(f"changed base_index: {b}")
-                    if num_changed is True:
-                        json.dump(num_index, file_num_index)
-                        logger.debug(f"changed file_num_index: {num_index_path}")
+        for b in base_indexes:
+            self._check_base_index_file(base_remove_count, b)
+
+        for n in num_indexes:
+            self._check_num_index_file(num_remove_count, n)
         logger.info((f"completed check of indexes \n"
                      f"base_index: {base_remove_count} entries removed \n"
                      f"num_index: {num_remove_count} entries removed \n"))
+
+    def _check_base_index_file(self, base_remove_count, b):
+        '''check if a file in the base index is missing and remove the entry if it is.'''
+        base_changed = False
+        original = pd.read_csv(b, delimiter=",")
+            # remove duplicates
+        df = original.drop_duplicates()
+        drop_rows = []
+            # load the file_num_index
+        for row in df.iloc:
+            if not (self.root_path /  "filings" / Path(row["file_path"])).is_file():
+                logger.debug(f"{(self.root_path / 'filings' / Path(row['file_path']))} didnt exist")
+                base_changed = True
+                drop_rows.append(row.name)
+        if base_changed is True:
+            base_remove_count += len(drop_rows)
+            df = df.drop(drop_rows)
+            df.to_csv(b)
+            logger.debug(f"changed base_index: {b}")
+
+    def _check_num_index_file(self, num_remove_count, n):
+        '''check if a file in the num index is missing and remove it if it is.'''
+        with open(n, "r+") as file_num_index:
+            num_changed =  False
+            num_index = json.load(file_num_index)
+            file_nums_to_remove = []
+            logger.debug(f"original num_index: {json.dumps(num_index, indent=2)}")
+                  # remove entry from file_num_index
+            for file_num in num_index.keys():
+                offset = 0
+                for idx, entry in enumerate(num_index[file_num]):
+                    if not (self.root_path / "filings" / Path(entry[1])).is_file():
+                        num_index[file_num].pop(idx - offset)
+                        offset += 1
+                        num_remove_count += 1
+                        num_changed = True
+                if num_index[file_num] == []:
+                    file_nums_to_remove.append(file_num)
+            if file_nums_to_remove != []:
+                for file_num_to_remove in file_nums_to_remove:
+                    del num_index[file_num_to_remove]
+        if num_changed is True:
+            with open(n, "w") as file_num_index:
+                logger.debug(f"changed num_index: {json.dumps(num_index, indent=2)}")
+                json.dump(num_index, file_num_index)
+            logger.debug(f"changed num_index: {n}")
     
     def _get_base_index_as_dataframe(self, cik: str):
         '''get the index of cik as a dataframe with absolute file paths and the accession number added'''
@@ -255,7 +269,7 @@ class IndexHandler:
         return self._base_index_path / cik10 +".csv"
     
     def _get_file_num_index_path(self, cik10):
-        return self._file_num_index_path / cik10 +".json"
+        return self._num_index_path / cik10 +".json"
 
     def _prepare_root_path(self, path: str | Path, create_folder=True):
         if not isinstance(path, Path) and isinstance(path, str):
@@ -285,15 +299,14 @@ class IndexHandler:
                 p = path.join(p, x)
         return p
     
-    
     def _create_indexes(self, cik: str, form_type: str, accn: str, file_name: str, file_num: str, filing_date: str):
         # need to fix rewritting whole json every time, how could i use seek()?
         '''create index files or add to them. accession number is included in the file_path'''
         if self._checked_index_creation is False:
             if not self._base_index_path.exists():
                 self._base_index_path.mkdir(parents=True)
-            if not self._file_num_index_path.exists():
-                self._file_num_index_path.mkdir(parents=True)
+            if not self._num_index_path.exists():
+                self._num_index_path.mkdir(parents=True)
             self._checked_index_creation = True
         rel_file_path = path.join(cik, form_type, accn, file_name)
         base_path = self._get_base_index_path(cik)
@@ -358,7 +371,7 @@ class IndexHandler:
         return self._base_index_path / (str(cik)+".csv")
     
     def _get_file_num_index_path(self, cik):
-        return self._file_num_index_path / (str(cik)+".json")
+        return self._num_index_path / (str(cik)+".json")
     
 
 class Downloader:
@@ -1166,3 +1179,4 @@ class Downloader:
         return session
 
 
+
